/*
pbcX.c
*/

#include "oth.h"
#include "complex.h"
#include "oth_prototype.h"

void copyXpbc(d_complex_t *v)
{
	assert(Pbc.xpbc);

	// Ex
	for (int j = j0Ex - 1; j <= j1Ex + 1; j++) {
	for (int k = k0Ex - 1; k <= k1Ex + 1; k++) {
		//assert(NEX(   - 1, j, k) >= 0 && NEX(   - 1, j, k) < NE);
		//assert(NEX(Nx + 0, j, k) >= 0 && NEX(Nx + 0, j, k) < NE);
		v[NEX(   - 1, j, k)] = v[NEX(Nx - 1, j, k)];
		v[NEX(Nx + 0, j, k)] = v[NEX(     0, j, k)];
	}
	}

	// Ey
	for (int j = j0Ey - 1; j <= j1Ey + 1; j++) {
	for (int k = k0Ey - 1; k <= k1Ey + 1; k++) {
		//assert(NEY(   - 1, j, k) >= 0 && NEY(   - 1, j, k) < NE);
		//assert(NEY(Nx + 1, j, k) >= 0 && NEY(Nx + 1, j, k) < NE);
		v[NEY(   - 1, j, k)] = v[NEY(Nx - 1, j, k)];
		v[NEY(Nx + 1, j, k)] = v[NEY(     1, j, k)];
	}
	}

	// Ez
	for (int j = j0Ez - 1; j <= j1Ez + 1; j++) {
	for (int k = k0Ez - 1; k <= k1Ez + 1; k++) {
		//assert(NEZ(   - 1, j, k) >= 0 && NEZ(   - 1, j, k) < NE);
		//assert(NEZ(Nx + 1, j, k) >= 0 && NEZ(Nx + 1, j, k) < NE);
		v[NEZ(   - 1, j, k)] = v[NEZ(Nx - 1, j, k)];
		v[NEZ(Nx + 1, j, k)] = v[NEZ(     1, j, k)];
	}
	}
}

void factorXpbc(d_complex_t *v)
{
	assert(Pbc.xpbc);
	int64_t n;
	int i;
	const d_complex_t fp = Pbc.fx;
	const d_complex_t fm = d_inv(fp);

	// Ex
	for (int j = j0Ex - 1; j <= j1Ex + 1; j++) {
	for (int k = k0Ex - 1; k <= k1Ex + 1; k++) {
		i =    - 1;
		if ((i >= i0Ex - 1) && (i <= i1Ex + 1)) {  // MPI
			n = NEX(i, j, k);
			v[n] = d_mul(v[n], fm);
		}

		i = Nx + 0;
		if ((i >= i0Ex - 1) && (i <= i1Ex + 1)) {  // MPI
			n = NEX(i, j, k);
			v[n] = d_mul(v[n], fp);
		}
	}
	}

	// Ey
	for (int j = j0Ey - 1; j <= j1Ey + 1; j++) {
	for (int k = k0Ey - 1; k <= k1Ey + 1; k++) {
		i =    - 1;
		if ((i >= i0Ey - 1) && (i <= i1Ey + 1)) {  // MPI
			n = NEY(i, j, k);
			v[n] = d_mul(v[n], fm);
		}

		i = Nx + 1;
		if ((i >= i0Ey - 1) && (i <= i1Ey + 1)) {  // MPI
			n = NEY(i, j, k);
			v[n] = d_mul(v[n], fp);
		}
	}
	}

	// Ez
	for (int j = j0Ez - 1; j <= j1Ez + 1; j++) {
	for (int k = k0Ez - 1; k <= k1Ez + 1; k++) {
		i =    - 1;
		if ((i >= i0Ez - 1) && (i <= i1Ez + 1)) {  // MPI
			n = NEZ(i, j, k);
			v[n] = d_mul(v[n], fm);
		}

		i = Nx + 1;
		if ((i >= i0Ez - 1) && (i <= i1Ez + 1)) {  // MPI
			n = NEZ(i, j, k);
			v[n] = d_mul(v[n], fp);
		}
	}
	}
}
